@@ -1,8 +1,7 @@
-import os
-import json
-import time
+import os, json
+import time, uuid
+from typing import Optional, List, Dict
 import openai
-from typing import Optional
 
 class ChatBot:
     """
@@ -13,33 +12,37 @@ class ChatBot:
         self,
         api_key: Optional[str] = None,
         engine: str = "gpt-3.5-turbo",
-        max_tokens: int = 3000,
+        # max_tokens: int = 3000,
         temperature: float = 0.5,
         top_p: float = 1.0,
         presence_penalty: float = 0.0,
         frequency_penalty: float = 0.0,
         proxy: str = None,
         max_turns: int = -1,
-        max_retries: int = 1,
+        max_retries: int = 3,
+        keep_round: int = 2,
+        system_prompt: str = "You are a helpful assistant.",
     ) -> None:
         """
         Initialize the Chatbot class.
 
         :param api_key: OpenAI API key, defaults to None.
         :param engine: GPT engine to use, defaults to "gpt-3.5-turbo".
-        :param max_tokens: Maximum number of tokens to generate, defaults to 3000.
+        # :param max_tokens: Maximum number of tokens to generate, defaults to 3000.
         :param temperature: Sampling temperature, lower values make the output more focused and deterministic, defaults to 0.5.
         :param top_p: The nucleus sampling parameter, set to 1.0 to disable, defaults to 1.0.
         :param presence_penalty: Controls the model's tendency to repeat itself, defaults to 0.0.
         :param frequency_penalty: Controls the model's use of less frequent words, defaults to 0.0.
         :param proxy: Optional proxy setting like "socks://<proxy_server>:<proxy_port>" or "http://<proxy_server:proxy_port>", defaults to None. 
         :param max_turns: Maximum number of turns per conversation, defaults to -1 for unlimited turns.
-        :param max_retries: Number of retries to ask the model if the response is empty, defaults to 1.
+        :param max_retries: Number of retries to ask the model if the response is empty, defaults to 3.
+        :param keep_round: Number of rounds to keep in the conversation history, defaults to 2.
+        :param system_prompt: The prompt for the system messages, defaults to "You are a helpful assistant."
         """
         self.engine = engine
         self.api_key = api_key or os.environ.get("OPENAI_API_KEY")
         openai.api_key = self.api_key
-        self.max_tokens = max_tokens or 3000
+        # self.max_tokens = max_tokens or 3000
         self.temperature = temperature
         self.top_p = top_p
         self.presence_penalty = presence_penalty
@@ -52,6 +55,10 @@ class ChatBot:
         self.system_contents = {}
         self.max_turns = max_turns
 
+        self.keep_round = keep_round
+        self.system_prompt = system_prompt
+        self.messages = [{"role": "system", "content": self.system_prompt}]
+
     def start_conversation(self, conversation_id: str = "default", system_content : str = "You are a helpful assistant.") -> None:
         """
         Start a new conversation with the given conversation_id.
@@ -63,7 +70,7 @@ class ChatBot:
             self.conversations[conversation_id] = []
             self.system_contents[conversation_id] = system_content 
     
-    def remove_conversation(self, conversation_id: str) -> None:
+    def remove_conversation(self, conversation_id: str = "default") -> None:
         """
         Remove a conversation from the conversation history.
 
@@ -117,7 +124,7 @@ class ChatBot:
         """
         return len(openai.api_utils.tokens_of_string(text))
 
-    def _chat_completion(self, prompt: str, role: str = "user", conversation_id: str = "default", stream: bool = False) -> openai.ChatCompletion:
+    def _chat_completion(self, prompt: str = None, messages: List[Dict[str, str]] = None, role: str = "user", conversation_id: str = "default", stream: bool = False) -> openai.ChatCompletion:
         """
         Generate a response for the given prompt.
         :param prompt: The user's input to generate a response for.
@@ -125,13 +132,16 @@ class ChatBot:
         :param conversation_id: The unique identifier for the conversation, defaults to "default".
         :return: The response from the model.
         """
+        assert prompt is not None or messages is not None, "Either prompt or messages must be provided."
 
         response = openai.ChatCompletion.create(
             model=self.engine,
-            messages=[
+            messages=messages or 
+            [
                 {
                     "role": "system",
-                    "content": self.system_contents[conversation_id]
+                    # "content": self.system_contents[conversation_id]
+                    "content": self.system_prompt
                 },
                 {
                     "role": role,
@@ -139,7 +149,7 @@ class ChatBot:
                 },
             ],
             temperature=self.temperature,
-            max_tokens=self.max_tokens,
+            # max_tokens=self.max_tokens,
             top_p=self.top_p,
             frequency_penalty=self.frequency_penalty,
             presence_penalty=self.presence_penalty,
@@ -167,21 +177,21 @@ class ChatBot:
             retries += 1
 
             try:
-                response = self._chat_completion(prompt, role=role, conversation_id=conversation_id, stream=True)
+                response = self._chat_completion(prompt = prompt, role=role, conversation_id=conversation_id, stream=True)
+                collected_messages = []
+                for chunk in response:
+                    chunk_message = chunk['choices'][0]['delta']
+                    collected_messages.append(chunk_message)
+                    yield chunk_message.get("content", '')
+        
+                reply_content = ''.join([m.get('content', '') for m in collected_messages])
+                self.add_message(reply_content, role="assistant", conversation_id=conversation_id)
             except openai.error.APIConnectionError as e:
                 print(f"Error: {e}")
                 print(f"Retrying... ({retries}/{self.max_retries})")
                 time.sleep(1)
                 continue
 
-        collected_messages = []
-        for chunk in response:
-            chunk_message = chunk['choices'][0]['delta']
-            collected_messages.append(chunk_message)
-            yield chunk_message.get("content", '')
-
-        reply_content = ''.join([m.get('content', '') for m in collected_messages])
-        self.add_message(reply_content, role="assistant", conversation_id=conversation_id)
 
 
     def ask(self, prompt: str, role: str = "user", conversation_id: str = "default", stream=False) -> str:
@@ -202,21 +212,83 @@ class ChatBot:
             retries += 1
 
             try:
-                response = self._chat_completion(prompt, role=role, conversation_id=conversation_id, stream=stream)
+                response = self._chat_completion(prompt=prompt, role=role, conversation_id=conversation_id, stream=stream)
+                if stream:
+                    unfinished_reply_content= ""
+                    for chunked_reply_content in self.ask_stream_iterator(prompt, conversation_id=conversation_id):
+                        unfinished_reply_content+= chunked_reply_content
+                        print(unfinished_reply_content)
+                    reply_content= self.get_last_reply_content(conversation_id)
+                else:
+                    reply_content = response.choices[0].message.content.strip()
+                    self.add_message(reply_content, role="assistant", conversation_id=conversation_id)
             except openai.error.APIConnectionError as e:
                 print(f"Error: {e}")
                 print(f"Retrying... ({retries}/{self.max_retries})")
                 time.sleep(1)
                 continue
 
-        if stream:
-            unfinished_reply_content= ""
-            for chunked_reply_content in self.ask_stream_iterator(prompt, conversation_id=conversation_id):
-                unfinished_reply_content+= chunked_reply_content
-                print(unfinished_reply_content)
-            reply_content= self.get_last_reply_content(conversation_id)
-        else:
-            reply_content = response.choices[0].message.content.strip()
-            self.add_message(reply_content, role="assistant", conversation_id=conversation_id)
 
-        return reply_content
\ No newline at end of file
+        return reply_content
+
+    def send_msg(self, messages: List[Dict[str,str]], role: str = "user", conversation_id: str = "default", stream: bool = False):
+        """
+        Send a message to the assistant.
+
+        :param msg: The message to send to the assistant.
+        :return: The assistant's response.
+        """
+        # if self.conversation_id is None:
+        #     self.conversation_id = str(uuid.uuid4())
+
+        # if self.conversation_turns == 0:
+        #     self.conversation_start_time = time.time()
+            
+        # self.conversation_turns += 1
+
+        # prompt = self._get_prompt(msg)
+        # reply = self._get_reply(prompt)
+
+        self.start_conversation(conversation_id)
+        # self.add_message(prompt, role=role, conversation_id=conversation_id)
+
+        retries = 0
+        while retries < self.max_retries:
+            retries += 1
+
+            try:
+                response = self._chat_completion(messages=messages, conversation_id=conversation_id, stream=stream)
+                reply_content = response.choices[0].message.content.strip()
+            except openai.error.APIConnectionError as e:
+                print(f"Error: {e}")
+                print(f"Retrying... ({retries}/{self.max_retries})")
+                time.sleep(1)
+                continue
+
+        return reply_content
+
+    def chat(self, message):
+        # This method is used to send a message and get a response from the OpenAI API
+        print(f"Total messages: {len(self.messages)}\nfirst_message: {self.messages[0]}\n")
+
+        # Adding the user message to the conversation messages
+        self.messages.append({"role": "user", "content": message})
+        # Sending the messages to the API and getting the response
+        response = self.send_msg(self.messages)
+        # Adding the system response to the conversation messages
+        self.messages.append({"role": "assistant", "content": response})
+        # Dropping previous conversation messages to keep the conversation history short
+        self.messages = self._drop_conversation(self.messages)
+        # Returning the system response
+        return response
+        
+    def _drop_conversation(self, msg):
+        # This method is used to drop previous messages from the conversation and keep only recent ones
+        if len(msg) >= (self.keep_round + 1) * 2 + 1:
+            # new_msg = [msg[0]]
+            new_msg = [{"role": "assistant", "content": self.system_prompt}]
+            for i in range(3, len(msg)):
+                new_msg.append(msg[i])
+            return new_msg
+        else:
+            return msg
\ No newline at end of file
